@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-:copyright: Copyright 2017-2020 Sphinx Confluence Builder Contributors (AUTHORS)
+:copyright: Copyright 2017-2022 Sphinx Confluence Builder Contributors (AUTHORS)
 :license: BSD-2-Clause (LICENSE)
 
 See also:
@@ -8,76 +8,266 @@ See also:
     https://docs.atlassian.com/confluence/REST/latest/
 """
 
+from sphinx.util.logging import skip_warningiserror
 from sphinxcontrib.confluencebuilder.exceptions import ConfluenceBadApiError
+from sphinxcontrib.confluencebuilder.exceptions import ConfluenceBadServerUrlError
 from sphinxcontrib.confluencebuilder.exceptions import ConfluenceBadSpaceError
 from sphinxcontrib.confluencebuilder.exceptions import ConfluenceConfigurationError
 from sphinxcontrib.confluencebuilder.exceptions import ConfluenceMissingPageIdError
 from sphinxcontrib.confluencebuilder.exceptions import ConfluencePermissionError
+from sphinxcontrib.confluencebuilder.exceptions import ConfluencePublishAncestorError
+from sphinxcontrib.confluencebuilder.exceptions import ConfluencePublishSelfAncestorError
 from sphinxcontrib.confluencebuilder.exceptions import ConfluenceUnreconciledPageError
-from sphinxcontrib.confluencebuilder.logger import ConfluenceLogger
+from sphinxcontrib.confluencebuilder.logger import ConfluenceLogger as logger
 from sphinxcontrib.confluencebuilder.rest import Rest
 import json
+import logging
 import time
 
-class ConfluencePublisher():
+
+class ConfluencePublisher:
     def __init__(self):
+        self.cloud = None
         self.space_display_name = None
+        self.space_type = None
+        self._ancestors_cache = set()
         self._name_cache = {}
 
-    def init(self, config):
+    def init(self, config, cloud=None):
+        self.cloud = cloud
         self.config = config
         self.append_labels = config.confluence_append_labels
-        self.can_labels = 'labels' not in config.confluence_adv_restricted
+        self.debug = config.confluence_publish_debug
         self.dryrun = config.confluence_publish_dryrun
         self.notify = not config.confluence_disable_notifications
         self.onlynew = config.confluence_publish_onlynew
         self.parent_id = config.confluence_parent_page_id_check
-        self.parent_name = config.confluence_parent_page
+        self.parent_ref = config.confluence_parent_page
         self.server_url = config.confluence_server_url
-        self.space_name = config.confluence_space_name
+        self.space_key = config.confluence_space_key
         self.watch = config.confluence_watch
 
         # append labels by default
         if self.append_labels is None:
             self.append_labels = True
 
+        # if debugging, enable requests (urllib3) logging
+        if self.debug:
+            logging.basicConfig()
+            logging.getLogger().setLevel(logging.DEBUG)
+            rlog = logging.getLogger('requests.packages.urllib3')
+            rlog.setLevel(logging.DEBUG)
+
     def connect(self):
         self.rest_client = Rest(self.config)
+        server_url = self.config.confluence_server_url
 
-        rsp = self.rest_client.get('space', {
-            'spaceKey': self.space_name,
-            'limit': 1
-        })
+        try:
+            rsp = self.rest_client.get('space', {
+                'spaceKey': self.space_key,
+                'limit': 1
+            })
+        except ConfluenceBadApiError as e:
+            raise ConfluenceBadServerUrlError(server_url, e)
+
+        # if no size entry is provided, this a non-Confluence API server
+        if 'size' not in rsp:
+            raise ConfluenceBadServerUrlError(server_url,
+                'server did not provide an expected response (no size)')
+
+        # handle if the provided space key was not found
         if rsp['size'] == 0:
+            if self.debug:
+                logger.info('''could not find the configured space
+
+(notice to debugging user)
+Either the space does not exist, or the user does not have permission to see
+the space. Another space search will be performed to sanity check the
+configuration to see if a similar space key exists, which can hint to a user
+that the space key may be misconfigured. If the following search request
+results in an access restriction, it is most likely that the authentication
+options are not properly configured, even if the previous search request
+reported a success (which can be permitted for anonymous users).
+''')
+
+            extra_desc = ''
+
+            # If the space key was not found, attempt to search for the space
+            # based off its descriptive name. If something is found, hint to the
+            # user to use a space's key value instead.
+            search_fields = {
+                'cql': 'type=space and space.title~"' + self.space_key + '"',
+                'limit': 2,
+            }
+            rsp = self.rest_client.get('search', search_fields)
+
+            if rsp['size'] == 1:
+                detected_space = rsp['results'][0]
+                space_key = detected_space['space']['key']
+                space_name = detected_space['title']
+                extra_desc = \
+                    '''\n\n''' \
+                    '''There appears to be a space '{0}' which has a name ''' \
+                    ''''{1}'. Did you mean to use this space?\n''' \
+                    '''\n''' \
+                    '''   confluence_space_key = '{0}'\n''' \
+                    ''''''.format(space_key, space_name)
+
+            elif rsp['size'] > 1:
+                extra_desc = \
+                    '''\n\n''' \
+                    '''Multiple spaces have been detected which use the ''' \
+                    '''name '{}'. The unique key of the space should be ''' \
+                    '''used instead. See also:\n\n''' \
+                    '''   https://support.atlassian.com/confluence-cloud/docs/choose-a-space-key/\n''' \
+                    ''''''.format(self.space_key)
+
             pw_set = bool(self.config.confluence_server_pass)
-            raise ConfluenceBadSpaceError(self.space_name,
-                self.config.confluence_server_user, pw_set)
-        self.space_display_name = rsp['results'][0]['name']
+            token_set = bool(self.config.confluence_publish_token)
+
+            raise ConfluenceBadSpaceError(
+                self.space_key,
+                self.config.confluence_server_user,
+                pw_set,
+                token_set,
+                extra_desc)
+
+        # sanity check that we have any result
+        if 'results' not in rsp or not rsp['results']:
+            raise ConfluenceBadServerUrlError(server_url,
+                'server did not provide an expected response (no results)')
+
+        result = rsp['results'][0]
+
+        if not isinstance(result, dict) or not result.get('name'):
+            raise ConfluenceBadServerUrlError(server_url,
+                'server did not provide an expected response (no name)')
+
+        # track required space information
+        self.space_display_name = result['name']
+        self.space_type = result['type']
 
     def disconnect(self):
         self.rest_client.close()
 
-    def getBasePageId(self):
+    def archive_page(self, page_id):
+        if self.dryrun:
+            self._dryrun('archive page', page_id)
+            return
+        elif self.onlynew:
+            self._onlynew('page archive restricted', page_id)
+            return
+
+        try:
+            data = {
+                'pages': [{'id': page_id}],
+            }
+
+            rsp = self.rest_client.post('content/archive', data)
+            longtask_id = rsp['id']
+
+            # wait for the archiving of the page to complete
+            MAX_WAIT_FOR_PAGE_ARCHIVE = 4  # ~2 seconds
+            attempt = 1
+            while attempt <= MAX_WAIT_FOR_PAGE_ARCHIVE:
+                time.sleep(0.5)
+
+                rsp = self.rest_client.get('longtask/{}'.format(longtask_id))
+                if rsp['finished']:
+                    break
+
+                attempt += 1
+                if attempt > MAX_WAIT_FOR_PAGE_ARCHIVE:
+                    raise ConfluenceBadApiError(
+                        -1, 'timeout waiting for archive completion')
+        except ConfluencePermissionError:
+            raise ConfluencePermissionError(
+                """Publish user does not have permission to archive """
+                """from the configured space."""
+            )
+
+    def archive_pages(self, page_ids):
+        if self.dryrun:
+            self._dryrun('archiving pages', ', '.join(page_ids))
+            return
+        elif self.onlynew:
+            self._onlynew('page archiving restricted', ', '.join(page_ids))
+            return
+
+        try:
+            data = {
+                'pages': [],
+            }
+
+            for page_id in page_ids:
+                data['pages'].append({'id': page_id})
+
+            # Note, multi-page archive can result in Confluence reporting the
+            # following message:
+            #  Cannot use bulk archive feature for non premium edition
+            self.rest_client.post('content/archive', data)
+
+        except ConfluencePermissionError:
+            raise ConfluencePermissionError(
+                """Publish user does not have permission to archive """
+                """from the configured space."""
+            )
+
+    def get_ancestors(self, page_id):
+        """
+        generate a list of ancestors
+
+        Queries the configured Confluence instance for a set of ancestors for
+        the provided `page_id`.
+
+        Args:
+            page_id: the ancestor to search on
+
+        Returns:
+            the ancestors
+        """
+
+        assert page_id
+        ancestors = set()
+
+        _, page = self.get_page_by_id(page_id, 'ancestors')
+
+        if 'ancestors' in page:
+            for ancestor in page['ancestors']:
+                ancestors.add(ancestor['id'])
+
+        return ancestors
+
+    def get_base_page_id(self):
         base_page_id = None
 
-        if not self.parent_name:
+        if not self.parent_ref:
+            return base_page_id
+
+        if isinstance(self.parent_ref, int):
+            base_page_id, page = self.get_page_by_id(self.parent_ref)
+
+            if not page:
+                raise ConfluenceConfigurationError(
+                    '''Configured parent page identifier does not exist.''')
+
             return base_page_id
 
         rsp = self.rest_client.get('content', {
             'type': 'page',
-            'spaceKey': self.space_name,
-            'title': self.parent_name,
+            'spaceKey': self.space_key,
+            'title': self.parent_ref,
             'status': 'current'
         })
         if rsp['size'] == 0:
-            raise ConfluenceConfigurationError("""Configured parent """
-                """page name do not exist.""")
+            raise ConfluenceConfigurationError(
+                '''Configured parent page name does not exist.''')
         page = rsp['results'][0]
         if self.parent_id and page['id'] != str(self.parent_id):
             raise ConfluenceConfigurationError("""Configured parent """
                 """page ID and name do not match.""")
         base_page_id = page['id']
-        self._name_cache[base_page_id] = self.parent_name
+        self._name_cache[base_page_id] = self.parent_ref
 
         if not base_page_id and self.parent_id:
             raise ConfluenceConfigurationError("""Unable to find the """
@@ -85,7 +275,7 @@ class ConfluencePublisher():
 
         return base_page_id
 
-    def getDescendants(self, page_id):
+    def get_descendants(self, page_id):
         """
         generate a list of descendants
 
@@ -103,7 +293,7 @@ class ConfluencePublisher():
         if page_id:
             search_fields = {'cql': 'ancestor=' + str(page_id)}
         else:
-            search_fields = {'cql': 'space="' + self.space_name +
+            search_fields = {'cql': 'space="' + self.space_key +
                 '" and type=page'}
 
         # Configure a larger limit value than the default (no provided
@@ -128,7 +318,7 @@ class ConfluencePublisher():
 
         return descendants
 
-    def getDescendantsCompat(self, page_id):
+    def get_descendants_compat(self, page_id):
         """
         generate a list of descendants (aggressive)
 
@@ -151,7 +341,7 @@ class ConfluencePublisher():
         visited_pages = set()
 
         def find_legacy_pages(page_id, pages):
-            descendants = self.getDescendants(page_id)
+            descendants = self.get_descendants(page_id)
             for descendant in descendants:
                 if descendant not in pages:
                     pages.add(descendant)
@@ -160,7 +350,7 @@ class ConfluencePublisher():
         find_legacy_pages(page_id, visited_pages)
         return visited_pages
 
-    def getAttachment(self, page_id, name):
+    def get_attachment(self, page_id, name):
         """
         get attachment information with the provided page id and name
 
@@ -191,7 +381,7 @@ class ConfluencePublisher():
 
         return attachment_id, attachment
 
-    def getAttachments(self, page_id):
+    def get_attachments(self, page_id):
         """
         get all known attachments for a provided page id
 
@@ -231,7 +421,7 @@ class ConfluencePublisher():
 
         return attachment_info
 
-    def getPage(self, page_name, expand='version'):
+    def get_page(self, page_name, expand='version'):
         """
         get page information with the provided page name
 
@@ -252,7 +442,7 @@ class ConfluencePublisher():
 
         rsp = self.rest_client.get('content', {
             'type': 'page',
-            'spaceKey': self.space_name,
+            'spaceKey': self.space_key,
             'title': page_name,
             'status': 'current',
             'expand': expand,
@@ -265,7 +455,7 @@ class ConfluencePublisher():
 
         return page_id, page
 
-    def getPageById(self, page_id, expand='version'):
+    def get_page_by_id(self, page_id, expand='version'):
         """
         get page information with the provided page name
 
@@ -275,13 +465,12 @@ class ConfluencePublisher():
         return ``None`` entries.
 
         Args:
-            page_name: the page name
+            page_id: the page identifier
             expand (optional): data to expand on
 
         Returns:
             the page id and page object
         """
-        page = None
 
         page = self.rest_client.get('content/{}'.format(page_id), {
             'status': 'current',
@@ -289,12 +478,12 @@ class ConfluencePublisher():
         })
 
         if page:
-            assert page_id == int(page['id'])
-            self._name_cache[page_id] = ['title']
+            assert int(page_id) == int(page['id'])
+            self._name_cache[page_id] = page['title']
 
         return page_id, page
 
-    def getPageCaseInsensitive(self, page_name):
+    def get_page_case_insensitive(self, page_name):
         """
         get page information with the provided page name (case-insensitive)
 
@@ -315,7 +504,7 @@ class ConfluencePublisher():
         page_id = None
 
         page_name = page_name.lower()
-        search_fields = {'cql': 'space="' + self.space_name +
+        search_fields = {'cql': 'space="' + self.space_key +
             '" and type=page and title~"' + page_name + '"'}
         search_fields['limit'] = 1000
 
@@ -325,7 +514,7 @@ class ConfluencePublisher():
             for result in rsp['results']:
                 result_title = result['title']
                 if page_name == result_title.lower():
-                    page_id, page = self.getPage(result_title)
+                    page_id, page = self.get_page(result_title)
                     break
 
             if page_id or rsp['size'] != rsp['limit']:
@@ -338,7 +527,7 @@ class ConfluencePublisher():
 
         return page_id, page
 
-    def storeAttachment(self, page_id, name, data, mimetype, hash, force=False):
+    def store_attachment(self, page_id, name, data, mimetype, hash_, force=False):
         """
         request to store an attachment on a provided page
 
@@ -353,7 +542,7 @@ class ConfluencePublisher():
             name: the attachment name
             data: the attachment data
             mimetype: the mime type of this attachment
-            hash: the hash of the attachment
+            hash_: the hash of the attachment
             force (optional): force publishing if exists (defaults to False)
 
         Returns:
@@ -362,7 +551,10 @@ class ConfluencePublisher():
         HASH_KEY = 'SCB_KEY'
         uploaded_attachment_id = None
 
-        _, attachment = self.getAttachment(page_id, name)
+        if self.dryrun:
+            attachment = None
+        else:
+            _, attachment = self.get_attachment(page_id, name)
 
         # check if attachment (of same hash) is already published to this page
         comment = None
@@ -372,11 +564,11 @@ class ConfluencePublisher():
                 comment = metadata['comment']
 
         if not force and comment:
-            parts = comment.split(HASH_KEY + ':')
+            parts = comment.split(HASH_KEY + ':', 1)
             if len(parts) > 1:
-                tracked_hash = parts[1]
-                if hash == tracked_hash:
-                    ConfluenceLogger.verbose('attachment ({}) is already '
+                tracked_hash = ''.join(parts[1].split())
+                if hash_ == tracked_hash:
+                    logger.verbose('attachment ({}) is already '
                         'published to document with same hash'.format(name))
                     return attachment['id']
 
@@ -393,8 +585,14 @@ class ConfluencePublisher():
 
         # publish attachment
         try:
+            # split hash comment into chunks to minimize rendering issues with a
+            # single one-world-long-hash value
+            hash_ = '{}:{}'.format(HASH_KEY, hash_)
+            chunked_hash = '\n'.join(
+                [hash_[i:i + 16] for i in range(0, len(hash_), 16)])
+
             data = {
-                'comment': '{}:{}'.format(HASH_KEY, hash),
+                'comment': chunked_hash,
                 'file': (name, data, mimetype),
             }
 
@@ -423,7 +621,7 @@ class ConfluencePublisher():
 
         return uploaded_attachment_id
 
-    def storePage(self, page_name, data, parent_id=None):
+    def store_page(self, page_name, data, parent_id=None):
         """
         request to store page information to a confluence instance
 
@@ -441,10 +639,10 @@ class ConfluencePublisher():
         uploaded_page_id = None
 
         if self.config.confluence_adv_trace_data:
-            ConfluenceLogger.trace('data', data['content'])
+            logger.trace('data', data['content'])
 
         if self.dryrun:
-            _, page = self.getPage(page_name, 'version,ancestors')
+            _, page = self.get_page(page_name, 'version,ancestors')
 
             if not page:
                 self._dryrun('adding new page ' + page_name)
@@ -463,10 +661,10 @@ class ConfluencePublisher():
                 return page['id']
 
         expand = 'version'
-        if self.can_labels and self.append_labels:
+        if self.append_labels:
             expand += ',metadata.labels'
 
-        _, page = self.getPage(page_name, expand=expand)
+        _, page = self.get_page(page_name, expand=expand)
 
         if self.onlynew and page:
             self._onlynew('skipping existing page', page['id'])
@@ -475,16 +673,14 @@ class ConfluencePublisher():
         try:
             # new page
             if not page:
-                newPage = self._buildPage(page_name, data)
-
-                if self.can_labels:
-                    self._populate_labels(newPage, data['labels'])
+                new_page = self._build_page(page_name, data)
+                self._populate_labels(new_page, data['labels'])
 
                 if parent_id:
-                    newPage['ancestors'] = [{'id': parent_id}]
+                    new_page['ancestors'] = [{'id': parent_id}]
 
                 try:
-                    rsp = self.rest_client.post('content', newPage)
+                    rsp = self.rest_client.post('content', new_page)
 
                     if 'id' not in rsp:
                         api_err = ('Confluence reports a successful page ' +
@@ -495,9 +691,17 @@ class ConfluencePublisher():
                                 rsp, indent=2))
                         except TypeError:
                             api_err += 'DATA: <not-or-invalid-json>'
-                        raise ConfluenceBadApiError(api_err)
+                        raise ConfluenceBadApiError(-1, api_err)
 
                     uploaded_page_id = rsp['id']
+
+                    # if we have labels and this is a non-cloud instance,
+                    # initial labels need to be applied in their own request
+                    labels = new_page['metadata']['labels']
+                    if not self.cloud and labels:
+                        url = 'content/{}/label'.format(uploaded_page_id)
+                        self.rest_client.post(url, labels)
+
                 except ConfluenceBadApiError as ex:
                     # Check if Confluence reports that the new page request
                     # fails, indicating it already exists. This is usually
@@ -508,10 +712,10 @@ class ConfluencePublisher():
                     if str(ex).find('title already exists') == -1:
                         raise
 
-                    ConfluenceLogger.verbose('title already exists warning '
+                    logger.verbose('title already exists warning '
                         'for page {}'.format(page_name))
 
-                    _, page = self.getPageCaseInsensitive(page_name)
+                    _, page = self.get_page_case_insensitive(page_name)
                     if not page:
                         raise
 
@@ -521,7 +725,7 @@ class ConfluencePublisher():
 
             # update existing page
             if page:
-                self._updatePage(page, page_name, data, parent_id=parent_id)
+                self._update_page(page, page_name, data, parent_id=parent_id)
                 uploaded_page_id = page['id']
 
         except ConfluencePermissionError:
@@ -535,7 +739,7 @@ class ConfluencePublisher():
 
         return uploaded_page_id
 
-    def storePageById(self, page_name, page_id, data):
+    def store_page_by_id(self, page_name, page_id, data):
         """
         request to store page information on the page with a matching id
 
@@ -554,7 +758,7 @@ class ConfluencePublisher():
             return page_id
 
         if self.dryrun:
-            _, page = self.getPageById(page_id)
+            _, page = self.get_page_by_id(page_id)
 
             if not page:
                 self._dryrun('unable to find page with id', page_id)
@@ -564,18 +768,18 @@ class ConfluencePublisher():
                 return page_id
 
         expand = 'version'
-        if self.can_labels and self.append_labels:
+        if self.append_labels:
             expand += ',metadata.labels'
 
         try:
-            _, page = self.getPageById(page_id, expand=expand)
+            _, page = self.get_page_by_id(page_id, expand=expand)
         except ConfluenceBadApiError as ex:
             if str(ex).find('No content found with id') == -1:
                 raise
-            raise ConfluenceMissingPageIdError(self.space_name, page_id)
+            raise ConfluenceMissingPageIdError(self.space_key, page_id)
 
         try:
-            self._updatePage(page, page_name, data)
+            self._update_page(page, page_name, data)
         except ConfluencePermissionError:
             raise ConfluencePermissionError(
                 """Publish user does not have permission to add page """
@@ -587,7 +791,7 @@ class ConfluencePublisher():
 
         return page_id
 
-    def removeAttachment(self, id):
+    def remove_attachment(self, id_):
         """
         request to remove an attachment
 
@@ -595,24 +799,24 @@ class ConfluencePublisher():
         attachment.
 
         Args:
-            id: the attachment
+            id_: the attachment
         """
         if self.dryrun:
-            self._dryrun('removing attachment', id)
+            self._dryrun('removing attachment', id_)
             return
         elif self.onlynew:
-            self._onlynew('attachment removal restricted', id)
+            self._onlynew('attachment removal restricted', id_)
             return
 
         try:
-            self.rest_client.delete('content', id)
+            self.rest_client.delete('content', id_)
         except ConfluencePermissionError:
             raise ConfluencePermissionError(
                 """Publish user does not have permission to delete """
                 """from the configured space."""
             )
 
-    def removePage(self, page_id):
+    def remove_page(self, page_id):
         if self.dryrun:
             self._dryrun('removing page', page_id)
             return
@@ -621,7 +825,18 @@ class ConfluencePublisher():
             return
 
         try:
-            self.rest_client.delete('content', page_id)
+            try:
+                self.rest_client.delete('content', page_id)
+            except ConfluenceBadApiError as ex:
+                if str(ex).find('Transaction rolled back') == -1:
+                    raise
+
+                with skip_warningiserror():
+                    logger.warn('delete failed; retrying...')
+                time.sleep(3)
+
+                self.rest_client.delete('content', page_id)
+
         except ConfluenceBadApiError as ex:
             # Check if Confluence reports that this content does not exist. If
             # so, we want to suppress the API error. This is most likely a
@@ -630,7 +845,7 @@ class ConfluencePublisher():
             if str(ex).find('No content found with id') == -1:
                 raise
 
-            ConfluenceLogger.verbose('ignore missing delete for page '
+            logger.verbose('ignore missing delete for page '
                 'identifier: {}'.format(page_id))
         except ConfluencePermissionError:
             raise ConfluencePermissionError(
@@ -638,7 +853,22 @@ class ConfluencePublisher():
                 """from the configured space."""
             )
 
-    def updateSpaceHome(self, page_id):
+    def restrict_ancestors(self, ancestors):
+        """
+        restrict the provided ancestors from being changed
+
+        Registers the provided set of ancestors from being used when page
+        updates will move the location of a page. This is a pre-check update
+        requests so that a page cannot be flagged as a descendant of itsel
+        (where Confluence self-hosted instances may not report an ideal error
+        message).
+
+        Args:
+            ancestors: the ancestors to check against
+        """
+        self._ancestors_cache = ancestors
+
+    def update_space_home(self, page_id):
         if not page_id:
             return
 
@@ -651,10 +881,11 @@ class ConfluencePublisher():
 
         page = self.rest_client.get('content/' + page_id, None)
         try:
-            self.rest_client.put('space', self.space_name, {
-                'key': self.space_name,
+            self.rest_client.put('space', self.space_key, {
+                'key': self.space_key,
                 'name': self.space_display_name,
-                'homepage': page
+                'homepage': page,
+                'type': self.space_type,
             })
         except ConfluencePermissionError:
             raise ConfluencePermissionError(
@@ -662,7 +893,7 @@ class ConfluencePublisher():
                 """space's homepage."""
             )
 
-    def _buildPage(self, page_name, data):
+    def _build_page(self, page_name, data):
         """
         build a page entity used for a new or updated page event
 
@@ -682,14 +913,24 @@ class ConfluencePublisher():
                     'value': data['content'],
                 }
             },
+            # only the legacy editor is supported at this time; forced v1
+            # since Confluence Cloud appears to have an inconsistent default
+            # editor
+            'metadata': {
+                'properties': {
+                    'editor': {
+                        'value': 'v1',
+                    }
+                }
+            },
             'space': {
-                'key': self.space_name,
+                'key': self.space_key,
             },
         }
 
         return page
 
-    def _updatePage(self, page, page_name, data, parent_id=None):
+    def _update_page(self, page, page_name, data, parent_id=None):
         """
         build a page update and publish it to the confluence instance
 
@@ -705,50 +946,78 @@ class ConfluencePublisher():
         """
         last_version = int(page['version']['number'])
 
-        updatePage = self._buildPage(page_name, data)
-        updatePage['id'] = page['id']
-        updatePage['version'] = {
+        update_page = self._build_page(page_name, data)
+        update_page['id'] = page['id']
+        update_page['version'] = {
             'number': last_version + 1,
+            'message': self.config.confluence_version_comment,
         }
 
-        if self.can_labels:
-            labels = list(data['labels'])
-            if self.append_labels:
-                labels.extend([lbl.get('name')
-                    for lbl in page.get('metadata', {}).get(
-                        'labels', {}).get('results', {})
-                ])
+        labels = list(data['labels'])
+        if self.append_labels:
+            labels.extend([lbl.get('name')
+                for lbl in page.get('metadata', {}).get(
+                    'labels', {}).get('results', {})
+            ])
 
-            self._populate_labels(updatePage, labels)
+        self._populate_labels(update_page, labels)
 
         if not self.notify:
-            updatePage['version']['minorEdit'] = True
+            update_page['version']['minorEdit'] = True
 
         if parent_id:
-            updatePage['ancestors'] = [{'id': parent_id}]
+            if page['id'] in self._ancestors_cache:
+                raise ConfluencePublishAncestorError(page_name)
 
+            update_page['ancestors'] = [{'id': parent_id}]
+
+            if page['id'] == parent_id:
+                raise ConfluencePublishSelfAncestorError(page_name)
+
+        page_id_explicit = page['id'] + '?status=current'
         try:
-            self.rest_client.put('content', page['id'], updatePage)
+            self.rest_client.put('content', page_id_explicit, update_page)
         except ConfluenceBadApiError as ex:
-            if str(ex).find('unreconciled') != -1:
-                raise ConfluenceUnreconciledPageError(
-                    page_name, page['id'], self.server_url, ex)
 
-            # Confluence Cloud may (rarely) fail to complete a
-            # content request with an OptimisticLockException/
-            # StaleObjectStateException exception. It is suspected
-            # that this is just an instance timing/processing issue.
-            # If this is observed, wait a moment and retry the
-            # content request. If it happens again, the put request
-            # will fail as it normally would.
-            if str(ex).find('OptimisticLockException') == -1:
+            # Handle select API failures by waiting a moment and retrying the
+            # content request. If it happens again, the put request will fail as
+            # it normally would.
+            retry_errors = [
+                # Confluence Cloud may (rarely) fail to complete a content
+                # request with an OptimisticLockException/
+                # StaleObjectStateException exception. It is suspected that this
+                # is just an instance timing/processing issue.
+                'OptimisticLockException',
+
+                # Confluence Cloud may (rarely) fail to complete a content
+                # request with an UnexpectedRollbackException exception. It is
+                # suspected that this is just a failed update event due to
+                # processing other updates.
+                'UnexpectedRollbackException',
+
+                # Confluence may report an unreconciled error -- either from a
+                # conflict with another instance updating the same page or some
+                # select backend issues processing previous updates on a page.
+                'unreconciled',
+            ]
+
+            if not any(x in str(ex) for x in retry_errors):
                 raise
-            ConfluenceLogger.warn(
-                'remote page updated failed; retrying...')
-            time.sleep(1)
-            self.rest_client.put('content', page['id'], updatePage)
 
-    def _dryrun(self, msg, id=None, misc=''):
+            with skip_warningiserror():
+                logger.warn('remote page updated failed; retrying...')
+            time.sleep(3)
+
+            try:
+                self.rest_client.put('content', page_id_explicit, update_page)
+            except ConfluenceBadApiError as ex:
+                if 'unreconciled' in str(ex):
+                    raise ConfluenceUnreconciledPageError(
+                        page_name, page['id'], self.server_url, ex)
+
+                raise
+
+    def _dryrun(self, msg, id_=None, misc=''):
         """
         log a dry run mode message
 
@@ -758,20 +1027,20 @@ class ConfluencePublisher():
 
         Args:
             msg: the message
-            id (optional): identifier (name mapping) associated with the message
+            id_ (optional): identifier (name mapping) associated with the message
             misc (optional): additional information to append
         """
         s = '[dryrun] '
         s += msg
-        if id and id in self._name_cache:
-            s += ' ' + self._name_cache[id]
-        if id:
-            s += ' ({})'.format(id)
+        if id_ and id_ in self._name_cache:
+            s += ' ' + self._name_cache[id_]
+        if id_:
+            s += ' ({})'.format(id_)
         if misc:
             s += ' ' + misc
-        ConfluenceLogger.info(s + min(80, 80 - len(s)) * ' ') # 80c-min clearing
+        logger.info(s + min(80, 80 - len(s)) * ' ')  # 80c-min clearing
 
-    def _onlynew(self, msg, id=None, misc=''):
+    def _onlynew(self, msg, id_=None):
         """
         log an only-new mode message
 
@@ -781,15 +1050,15 @@ class ConfluencePublisher():
 
         Args:
             msg: the message
-            id (optional): identifier (name mapping) associated with the message
+            id_ (optional): identifier (name mapping) associated with the message
         """
         s = '[only-new] '
         s += msg
-        if id and id in self._name_cache:
-            s += ' ' + self._name_cache[id]
-        if id:
-            s += ' ({})'.format(id)
-        ConfluenceLogger.info(s + min(80, 80 - len(s)) * ' ') # 80c-min clearing
+        if id_ and id_ in self._name_cache:
+            s += ' ' + self._name_cache[id_]
+        if id_:
+            s += ' ({})'.format(id_)
+        logger.info(s + min(80, 80 - len(s)) * ' ')  # 80c-min clearing
 
     def _populate_labels(self, page, labels):
         """
